import unittest

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

import ldap3
from ldap3.utils.dn import safe_dn
from ldap3.operation.search import parse_filter
from ldap3 import ALL_ATTRIBUTES

from .data import directory


# This replaces the original _execute_search implementation so that it fetches ldap
# attributes that have option extenstions, e.g. x-LIGO-uid;x-ligo-ldg-cdf
def _execute_search(self, request):
    from ldap3.utils.conv import to_unicode
    from ldap3.core.results import RESULT_NO_SUCH_OBJECT

    responses = []
    base = safe_dn(request["base"])
    scope = request["scope"]
    attributes = request["attributes"]
    if "+" in attributes:  # operational attributes requested
        attributes.extend(self.operational_attributes)
        attributes.remove("+")
    attributes = [attr.lower() for attr in request["attributes"]]

    filter_root = parse_filter(
        request["filter"],
        self.connection.server.schema,
        auto_escape=True,
        auto_encode=False,
        validator=self.connection.server.custom_validator,
        check_names=self.connection.check_names,
    )
    candidates = []
    if scope == 0:  # base object
        if base in self.connection.server.dit or base.lower() == "cn=schema":
            candidates.append(base)
    elif scope == 1:  # single level
        for entry in self.connection.server.dit:
            if (
                entry.endswith(base) and "," not in entry[: -len(base) - 1]
            ):  # only leafs without commas in the remaining dn
                candidates.append(entry)
    elif scope == 2:  # whole subtree
        for entry in self.connection.server.dit:
            if entry.endswith(base):
                candidates.append(entry)

    if not candidates:  # incorrect base
        result_code = RESULT_NO_SUCH_OBJECT
        message = "incorrect base object"
    else:
        matched = self.evaluate_filter_node(filter_root, candidates)
        for match in matched:
            responses.append(
                {
                    "object": match,
                    "attributes": [
                        {
                            "type": attribute,
                            "vals": []
                            if request["typesOnly"]
                            else self.connection.server.dit[match][attribute],
                        }
                        for attribute in self.connection.server.dit[match]
                        # This is the code that has been patched to return the attribute
                        # irrespective of the attribute-option
                        if attribute.lower().split(";")[0] in attributes
                        or ALL_ATTRIBUTES in attributes
                    ],
                }
            )

        result_code = 0
        message = ""

    result = {
        "resultCode": result_code,
        "matchedDN": "",
        "diagnosticMessage": to_unicode(message, "utf-8"),
        "referral": None,
    }

    return (
        responses[: request["sizeLimit"]] if request["sizeLimit"] > 0 else responses,
        result,
    )


class MockConnection(ldap3.Connection):
    def __init__(self, *args, **kwargs):
        kwargs["client_strategy"] = ldap3.MOCK_SYNC
        super(MockConnection, self).__init__(*args, **kwargs)

        # Mock connections cannot use auto_bind
        self.bind()

        # Populate connection with items from directory
        for dn, entry in self.directory.items():
            self.strategy.add_entry(dn, entry)

        # Monkey patch the search function to return attributes with options
        # Not sure I need to wrap this, but it doesn't work otherwise
        def wrap_execute_search(request):
            return _execute_search(self.strategy, request)

        self.strategy._execute_search = wrap_execute_search


class MockLIGOLDAP(unittest.TestCase):
    # default LDAP directory
    directory = directory

    @classmethod
    def setUpClass(cls):
        # Specifc mock connection for this classes LDAP directory
        class ThisMockConnection(MockConnection):
            directory = cls.directory

        # Patch the normal connection object with our specific one
        patch("ldap3.Connection", ThisMockConnection).start()
